package com.willalp.web.controller.common;

import com.willalp.common.config.WillalpConfig;
import com.willalp.common.constant.Constants;
import com.willalp.common.core.domain.AjaxResult;
import com.willalp.common.exception.file.FileNameLengthLimitExceededException;
import com.willalp.common.utils.StringUtils;
import com.willalp.common.utils.file.FileUploadUtils;
import com.willalp.common.utils.file.FileUtils;
import com.willalp.framework.config.ServerConfig;
import com.willalp.oss.utils.AttachmentHelper;
import org.apache.commons.fileupload.FileUploadBase;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;

/**
 * 通用请求处理
 *
 * @author willalp
 */
@RestController
public class CommonController {
    private static final Logger log = LoggerFactory.getLogger(CommonController.class);

    @Autowired
    private ServerConfig serverConfig;
    @Autowired
    private AttachmentHelper attachmentHelper;

    /**
     * 通用下载请求
     *
     * @param fileName 文件名称
     * @param delete 是否删除
     */
    @GetMapping("common/download")
    public void fileDownload(String fileName, Boolean delete, HttpServletResponse response, HttpServletRequest request) {
        try {
            if (!FileUtils.checkAllowDownload(fileName)) {
                throw new Exception(StringUtils.format("文件名称({})非法，不允许下载。 ", fileName));
            }
            String realFileName = System.currentTimeMillis() + fileName.substring(fileName.indexOf("_") + 1);
            String filePath = WillalpConfig.getDownloadPath() + fileName;

            response.setContentType(MediaType.APPLICATION_OCTET_STREAM_VALUE);
            FileUtils.setAttachmentResponseHeader(response, realFileName);
            FileUtils.writeBytes(filePath, response.getOutputStream());
            if (delete) {
                FileUtils.deleteFile(filePath);
            }
        } catch (Exception e) {
            log.error("下载文件失败", e);
        }
    }

    /**
     * 通用上传请求
     */
    @PostMapping("/common/upload")
    public AjaxResult uploadFile(MultipartFile file) throws Exception {
        try {
            // 上传文件路径
            String filePath = WillalpConfig.getUploadPath();
            // 上传并返回新文件名称
            String fileName = file.getOriginalFilename();
            String url = serverConfig.getUrl() + FileUploadUtils.upload(filePath, file);
            AjaxResult ajax = AjaxResult.success();
            ajax.put("fileName", fileName);
            ajax.put("url", url);
            return ajax;
        } catch (Exception e) {
            return AjaxResult.error(e.getMessage());
        }
    }

    /**
     * 本地资源通用下载
     */
    @GetMapping("/common/download/resource")
    public void resourceDownload(String resource, HttpServletRequest request, HttpServletResponse response)
            throws Exception {
        try {
            if (!FileUtils.checkAllowDownload(resource)) {
                throw new Exception(StringUtils.format("资源文件({})非法，不允许下载。 ", resource));
            }
            // 本地资源路径
            String localPath = WillalpConfig.getProfile();
            // 数据库资源地址
            String downloadPath = localPath + StringUtils.substringAfter(resource, Constants.RESOURCE_PREFIX);
            // 下载名称
            String downloadName = StringUtils.substringAfterLast(downloadPath, "/");
            response.setContentType(MediaType.APPLICATION_OCTET_STREAM_VALUE);
            FileUtils.setAttachmentResponseHeader(response, downloadName);
            FileUtils.writeBytes(downloadPath, response.getOutputStream());
        } catch (Exception e) {
            log.error("下载文件失败", e);
        }
    }

    @RequestMapping(value = "/common/ossUpload", method = RequestMethod.POST)
    public AjaxResult upload(HttpServletRequest request, MultipartFile file, @RequestParam(required = false, defaultValue = "") String dir) {
        try {
            return AjaxResult.success((Object) attachmentHelper.upload(request, file, dir));
        } catch (IOException e) {
            return AjaxResult.error("上传失败");
        } catch (FileUploadBase.FileSizeLimitExceededException e) {
            return AjaxResult.error("上传失败");
        } catch (FileNameLengthLimitExceededException e) {
            return AjaxResult.error("上传失败");
        } catch (com.willalp.oss.exception.FileNameLengthLimitExceededException e) {
            return AjaxResult.error("上传失败");
        } catch (com.willalp.oss.exception.InvalidExtensionException e) {
            return AjaxResult.error("上传失败");
        }
    }

}
